@@ -185,7 +185,7 @@ sub detach_db {
             delete $h_ref->{$plugin}{$db};
             if ( ! %{$h_ref->{$plugin}} ) {
                 delete $h_ref->{$plugin};
-                if ( ! %$h_ref ) {
+                if ( ! %$h_ref ) { # ###
                     if ( -f $file_fs && ! eval { unlink $file_fs or die "unlink '$file_fs': $!" } ) {
                         $ax->print_error_message( $@ );
                     }
