@@ -198,7 +198,7 @@ sub create_table {
                         my $data_types = {}; # data_types memory
 
                         EDIT_COLUMN_TYPES: while( 1 ) {
-                            $data_types = $sf->__edit_column_types( $sql, $data_types );
+                            $data_types = $sf->__edit_column_types( $sql, $source, $data_types );
                             if ( ! $data_types ) {
                                 $sql->{ct_column_definitions} = [ @unquoted_ct_column_definitions ];
                                 $sql->{insert_col_names} = [ @unquoted_insert_col_names ];
@@ -363,15 +363,15 @@ sub __primary_key_autoincrement_constraint {
     # provide "primary_key_autoincrement_constraint" only if also "first_col_is_autoincrement" in InsertUpdateDelete.pm is available
     my ( $sf ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $driver = $sf->{i}{driver};
-    if ( $driver eq 'SQLite' ) {
-        return "INTEGER PRIMARY KEY NOT NULL";
+    my $dbms = $sf->{i}{dbms};
+    if ( $dbms eq 'SQLite' ) {
+        return "INTEGER PRIMARY KEY";
     }
-    if ( $driver =~ /^(?:mysql|MariaDB)\z/ ) {
+    if ( $dbms =~ /^(?:mysql|MariaDB)\z/ ) {
         return "INT NOT NULL AUTO_INCREMENT PRIMARY KEY";
         # mysql: NOT NULL added automatically with AUTO_INCREMENT
     }
-    if ( $driver eq 'Pg' ) {
+    if ( $dbms eq 'Pg' ) {
         my $pg_major_version = $ax->major_server_version();
         if ( $pg_major_version >= 10 ) {
             # since PostgreSQL version 10
@@ -381,7 +381,7 @@ sub __primary_key_autoincrement_constraint {
             return "SERIAL PRIMARY KEY";
         }
     }
-    if ( $driver eq 'Firebird' ) {
+    if ( $dbms eq 'Firebird' ) {
         my $firebird_major_version = $ax->major_server_version();
         if ( $firebird_major_version >= 4 ) {
             return "INT GENERATED ALWAYS AS IDENTITY PRIMARY KEY";
@@ -390,15 +390,18 @@ sub __primary_key_autoincrement_constraint {
             return "INT GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY";
         }
     }
-    if ( $driver eq 'DB2' ) {
+    if ( $dbms eq 'DB2' ) {
         return "INT NOT NULL GENERATED ALWAYS AS IDENTITY PRIMARY KEY";
     }
-    if ( $driver eq 'Oracle' ) {
+    if ( $dbms eq 'Oracle' ) {
         my $ora_major_server_version = $ax->major_server_version();
         if ( $ora_major_server_version >= 12 ) {
             return "NUMBER GENERATED ALWAYS AS IDENTITY";   # Oracle 12c or greater
         }
     }
+    if ( $dbms eq 'MSSQL' ) {
+        return "INT IDENTITY(1,1) PRIMARY KEY";
+    }
 }
 
 
@@ -431,23 +434,32 @@ sub __edit_column_names {
 
 
 sub __edit_column_types {
-    my ( $sf, $sql, $data_types ) = @_;
+    my ( $sf, $sql, $source, $data_types ) = @_;
     my $tf = Term::Form->new( $sf->{i}{tf_default} );
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $driver = $sf->{i}{driver};
+    my $dbms = $sf->{i}{dbms};
     my $fields;
     if ( ! %$data_types && $sf->{o}{create}{data_type_guessing} ) {
         $ax->print_sql_info( $ax->get_sql_info( $sql ), 'Column data types: guessing ... ' );
-        require SQL::Type::Guess;
-        my $g = SQL::Type::Guess->new();
         my $header = $sql->{insert_col_names}; #
         my $table  = $sql->{insert_args};
-        my @aoh;
-        for my $row ( @$table ) {
-            push @aoh, {
-                map { $header->[$_] => $row->[$_] } 0 .. $#$header
-            };
+        my $encoding;
+        if ( $dbms eq 'MSSQL' && $sf->{o}{create}{encode_for_data_type_guessing} ) { # ###
+            $encoding = $source->{source_type} eq 'plain' ? 'locale' : $sf->{o}{insert}{file_encoding};
         }
+        my @aoh;
+        if ( length $encoding ) {
+            for my $row ( @$table ) {
+                push @aoh, { map { $header->[$_] => Encode::encode( $encoding, $row->[$_] ) } 0 .. $#$header };
+            }
+        }
+        else {
+            for my $row ( @$table ) {
+                push @aoh, { map { $header->[$_] => $row->[$_] } 0 .. $#$header };
+            }
+        }
+        require SQL::Type::Guess;
+        my $g = SQL::Type::Guess->new();
         $g->guess( @aoh );
         my $tmp = $g->column_type;
         $data_types = { map { $_ => uc( $tmp->{$_} ) } keys %$tmp };
@@ -463,16 +475,25 @@ sub __edit_column_types {
         unshift @$fields, [ $sql->{ct_column_definitions}[0], $sf->__primary_key_autoincrement_constraint() ];
         $read_only = [ 0 ];
     }
-    if ( $driver =~ /^(?:Pg|DuckDB|Firebird|Informix|Oracle)\z/ ) {
+    if ( $dbms =~ /^(?:Pg|DuckDB|Firebird|Informix|Oracle)\z/ ) {
         for my $field ( @$fields ) {
             if ( defined $field->[1] && $field->[1] eq 'DATETIME' ) {
-                $field->[1] = 'TIMESTAMP'                 if $driver =~ /^(?:Pg|DuckDB|Firebird)\z/;
-                $field->[1] = 'DATE'                      if $driver eq 'Oracle';
-                $field->[1] = 'DATETIME YEAR TO FRACTION' if $driver eq 'Informix';
+                $field->[1] = 'TIMESTAMP'                 if $dbms =~ /^(?:Pg|DuckDB|Firebird)\z/;
+                $field->[1] = 'DATE'                      if $dbms eq 'Oracle';
+                $field->[1] = 'DATETIME YEAR TO FRACTION' if $dbms eq 'Informix';
                 # Informix: DATETIME largest_qualifier TO smallest_qualifier
             }
         }
     }
+
+    if ( $dbms eq 'MSSQL' && $sf->{o}{create}{decimal_precision_includes_dot} ) { # ###
+        for my $field ( @$fields ) {
+            if ( defined $field->[1] && $field->[1] =~ /^DECIMAL\(([1-9][0-9]*),([1-9]+)\)\z/ ) {
+                $field->[1] = 'DECIMAL(' . ( $1 + 1 ) . ",$2)";
+            }
+        }
+    }
+
     my $constraint_rows = $sf->{o}{create}{table_constraint_rows};
     my $tbl_option_rows = $sf->{o}{create}{table_option_rows};
     my $skip = ' ';
